#include "SendEmail.h"

#include <windows.h>

#pragma comment(lib,"ws2_32.lib")				// Winsock


CSendEmail::CSendEmail( const char *szOutServer, const char *szInServer, const char *szUserName, const char *szPassword, const char *szFrom ) 
	:m_sOutServer(szOutServer), m_sInServer(szInServer), m_sFrom(szFrom), m_sUserName(szUserName), m_sPassword(szPassword), m_bWinSockAvail(false)
{
	// init the winsock libraries
	WSADATA wd;

	if( WSAStartup(MAKEWORD(1,1), &wd) != 0 )
		m_sResponse +="CSendEmail Error: Unable to initialize WinSock 1.1\n";
	else
		m_bWinSockAvail=true;
}




void CSendEmail::ReceiveLine( SOCKET connection )
{
	char szBuffer[1024+1];		//+1 for 0 termination
	int y=recv(connection,szBuffer,1024,0);
	
	szBuffer[y]=0;

	m_sResponse += szBuffer;
}

void CSendEmail::SendLine( SOCKET connection, const char *format, ... )
{
	static char szToSend[2048];
	va_list	ap;
	va_start(ap, format);
	vsprintf(szToSend, format, ap);
	va_end(ap);

	send(connection,szToSend,strlen(szToSend),0);
}

CSendEmail::~CSendEmail()
{
	if(m_bWinSockAvail)
		WSACleanup();
}


SOCKET CSendEmail::Open( const char *szHost, unsigned short port,	sockaddr_in &server_address )
{
	SOCKET connection;

	connection = socket(AF_INET,SOCK_STREAM,IPPROTO_TCP);
	if ( connection == INVALID_SOCKET )
	{
		m_sResponse += "CSendEmail Error creating socket(), WSAStartup() call missing?\n";
		return 0;
	}

	struct hostent *host_entry;

	host_entry=gethostbyname(m_sOutServer.c_str());

	if(!host_entry)
	{
		m_sResponse += "CSendEmail Error host(outgoing) '";
		m_sResponse += m_sOutServer;
		m_sResponse += "' not found\n";
		closesocket(connection);
		return 0;
	}

	server_address.sin_addr.s_addr =*((unsigned long*)host_entry->h_addr);
	
	server_address.sin_family=AF_INET;
	server_address.sin_port=htons(port);

	return connection;
}



// more on pop3, smtp, imap:
// http://www.hackerboard.de/thread.php?postid=242127&sid=80a309c94292418f0a588791ef7f1955#post242127
// http://www.aspheute.com/artikel/20001009.htm
// http://computer-programming-tutorials.suite101.com/article.cfm/sending_email_with_smtp_auth

bool CSendEmail::Send( const char *szTo, const char *szSubject, const char *szContent )
{
	if(!m_bWinSockAvail)
		return false;

	if(!LoginLogout())		// to ensure Server knows us and forward the email for us, this is to prevent spam
		return false;

	sockaddr_in server_address;
	SOCKET connection = Open(m_sOutServer.c_str(),25,server_address);		// port 25 (SMTP telnet)

	if(connection==INVALID_SOCKET)
		return false;

	if(connect(connection,(struct sockaddr*)&server_address,sizeof(server_address))!= SOCKET_ERROR)
	{
		ReceiveLine(connection);		// that can take a long time (seconds)

		SendLine(connection,"HELO %s\r\n",m_sInServer.c_str());			// imap 25
		ReceiveLine(connection);

		SendLine(connection,"MAIL From: %s\r\n",m_sFrom.c_str());
		ReceiveLine(connection);
		SendLine(connection,"RCPT To: %s\r\n",szTo);
		ReceiveLine(connection);

		SendLine(connection,"DATA\r\n");
		ReceiveLine(connection);

		SendLine(connection,"Subject: %s\r\n",szSubject);

		SendLine(connection,"From: %s\r\n",m_sFrom.c_str());		// redundant, can be used to fake from
		SendLine(connection,"To: %s\r\n",szTo);									// redundant, can be used to fake to

		SendLine(connection,"%s",szContent);

		SendLine(connection,"\r\n");
		SendLine(connection,".\r\n");
		ReceiveLine(connection);
		SendLine(connection,"quit\r\n");
		ReceiveLine(connection);
	}
	else
	{
		m_sResponse += "CSendEmail Error: connect failed\n";
		return false;
	}

	closesocket(connection);
	return true;
}





bool CSendEmail::LoginLogout()
{
	sockaddr_in server_address;
	SOCKET connection = Open(m_sInServer.c_str(),110,server_address);		// port 25 (SMTP telnet)

	if(connection==INVALID_SOCKET)
		return false;

	if (connect(connection,(struct sockaddr*)&server_address,sizeof(server_address))!= SOCKET_ERROR)
	{
		ReceiveLine(connection);

		SendLine(connection,"user %s\r\n",m_sUserName.c_str());		// pop3 110
		ReceiveLine(connection);
		SendLine(connection,"pass %s\r\n",m_sPassword.c_str());		// pop3 110
		ReceiveLine(connection);
		SendLine(connection,"quit\r\n");
		ReceiveLine(connection);
	}
	else
	{
		m_sResponse += "CSendEmail Error: connect failed\n";
		return false;
	}

	closesocket(connection);
	return true;
}