#pragma once

// CSendEmail, V:1.0
//
// Info: a simple class to send emails through POP3 from C++, uses Winsock, Windows, does not require any email client to be installed
// Additional info: code was written with tab size 2, compiled with Visual Studio
// Author: Martin Mittring (Kosmokleaner@Kosmokleaner.de)
// Copyright: code is free to use in any way, use at your own risk
// Files: CSendEmail.cpp, CSendEmail.h
// Known issues:
//  * Testing with other servers might be required
//  * Error handling could be improved
//  * Specific content (newline,".") could be used to stop message and do worse

#include <string>					// STL string
#include <windows.h>			// SOCKET

// e.g.
// 
// CSendEmail email("Out.going.Server","In.coming.Server","Username","Password","From@From.de");
//
// if(!email.Send("To@To.de","3emailsubject","3mycontent\r\nline2\r\nline3"))
//   Error();
//
class CSendEmail
{
public:
	//! constructor
	//! e.g. CSendEmail email("post.strato.de","post.strato.de","Username","Password","Sender@Sender.de");
	CSendEmail( const char *szOutServer, const char *szInServer, const char *szUserName, const char *szPassword, const char *szFrom );

	//! destructor
	~CSendEmail();

	//! e.g. email.Send("Receiver@Receiver.de","emailsubject","mycontent\r\nline2\r\nline3");
	bool Send( const char *szTo, const char *szSubject, const char *szContent );

	//! \return don't store pointer
	const char *GetResponse() { return m_sResponse.c_str(); }

private: // --------------------------------------------------------------

	bool										m_bWinSockAvail;	//!< true=winsock available, false=error
	std::string							m_sOutServer;			//!<
	std::string							m_sInServer;			//!<
	std::string							m_sFrom;					//!< from email adress
	std::string							m_sUserName;			//!< username
	std::string							m_sPassword;			//!<

	std::string							m_sResponse;			//!<

private: // --------------------------------------------------------------

	//! \return INVALID_SOCKET in case of error
	SOCKET Open( const char *szHost, unsigned short port, sockaddr_in &server_address );

	//!
	void SendLine( SOCKET connection, const char *format, ... );
	//!
	void ReceiveLine( SOCKET connection );
	//! to ensure Server knows us and forward the email for us
	bool LoginLogout();
};
